﻿//******************************************************************************
//Name          : FIRfromIDFT.cs
//Description   : Implementation to experience the beauty and the pitfalls of Filtering
//Author        : Dr. Oliver Kluge, 90411 Nuernberg, Germany
//Copyright(C)  : Oliver Kluge 2014 
//------------------------------------------------------------------------------
//******************************************************************************

using System;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;
using Display;


class FIRFilter
{
    const int samplerate = 128;                     // Abtastwerte je Periode
    const double PI      =   3.141592653589793;

    public double[] a = new double[samplerate];	    // Filter kernel in the time domain
    public double[] AS = new double[samplerate];    // Amplitudenspektrum
    public double[] AS2 = new double[samplerate];   // Amplitudenspektrum
    public double[] PS = new double[samplerate];    // Phasenspektrum
    public double[] arg = new double[samplerate];

    public int filterlength;

    public int Samplerate
    {
        get { return samplerate; }
    }

    public void CreateSignal(int select)
    {
        if (select >= 1 && select <= 8)
        {
            int width = 0;

            // 1/1 Puls
            if (select == 1) width = samplerate;
            // 1/2 Puls
            if (select == 2) width = samplerate / 2;
            // 1/4 Puls
            if (select == 3) width = samplerate / 4;
            // 1/8 Puls
            if (select == 4) width = samplerate / 8;
            // 1/16 Puls
            if (select == 5) width = samplerate / 16;
            // 1/32 Puls
            if (select == 6) width = samplerate / 32;
            // 1/64 Puls
            if (select == 7) width = samplerate / 64;
            // 1/128 Puls
            if (select == 8) width = samplerate / 128;

            for (int k = 0; k < samplerate; k++)
            {
                if (k < width)
                {
                    AS[k] = 1;
                }

                // to run the IDFT properly, we need to mirror the signal
                // to the upper frequency band above f=samplerate/2
                if (k < samplerate/2)
                {
                    // XS[127]=XS[1], XS[126]=XS[2], XS[125]=XS[3], ...
                    AS[samplerate-k-1] = AS[k];
                }
            }
        }

        // store argument data
        for (int k = 0; k < samplerate; k++)
        {
            arg[k] = k;
        }
    }

    public void IDFT()
    {
        double real = 0;
        double imag = 0;
        int phase = samplerate/2;

        for (int k = 0; k < samplerate; k++)
        {
            real = 0;
            imag = 0;

            for (int f = 0; f < samplerate; f++)
            {
                real += (AS[f] * Math.Cos(PS[f])) * Math.Cos(2*PI*(k+phase)*f/samplerate);
                imag += (AS[f] * Math.Sin(PS[f])) * Math.Sin(2*PI*(k+phase)*f/samplerate);
            }
            real /= 2.0;
            imag /= 2.0;

            a[k] = real + imag;// +AS[0]/2.0;     // filter coefficients
        }
    }

    public void FrequencyResponse(double startFreq, double stopFreq)
    {
        double deltaf = (stopFreq-startFreq)/samplerate;
        double freq   = startFreq;
        double real = 0;
        double imag = 0;

        for (int i=0; i<samplerate; i++)
        {
            real = 0;
            imag = 0;

            for (int k=0; k<samplerate; k++)
            {
                real += a[k]*Math.Cos(2*PI*freq*k/samplerate);
                imag += a[k]*Math.Sin(2*PI*freq*k/samplerate);
            }
            real *= 2.0 / samplerate;
            imag *= 2.0 / samplerate;

            AS2[i] = Math.Sqrt(real*real + imag*imag);
            PS[i] = Math.Atan2(imag, real);

            freq += deltaf;
        }
    }
}


class MainApp
{
    [STAThread]
    static void Main()
    {
        string userinput = "";
        int select = 0;

        while (select <= 17)
        {
            FIRFilter myFilter = new FIRFilter();

            Console.Write("Please select the desired frequency response:\n\n");
            Console.Write("1/1   Puls    ->  1\n");
            Console.Write("1/2   Puls    ->  2\n");
            Console.Write("1/4   Puls    ->  3\n");
            Console.Write("1/8   Puls    ->  4\n");
            Console.Write("1/16  Puls    ->  5\n");
            Console.Write("1/32  Puls    ->  6\n");
            Console.Write("1/64  Puls    ->  7\n");
            Console.Write("1/128 Puls    ->  8\n");

            Console.WriteLine();
            userinput = Console.ReadLine();
            select = Convert.ToInt32(userinput);

            // first of all we create the desired frequency response in the frequency domain
            myFilter.CreateSignal(select);

            // then we calculate the time domain filter kernel by using the IDFT
            myFilter.IDFT();

            // at least we calculate the frequency response of our calculated filter kernel
            myFilter.FrequencyResponse(0, myFilter.Samplerate/8);

            // create a chart for displaying that data...
            DisplayChart mydisplay1 = new DisplayChart(720, 480, SeriesChartType.Line, "Desired frequency response", "f", "|H(f)|");
            mydisplay1.AddData(myFilter.arg, myFilter.AS);

            DisplayChart mydisplay2 = new DisplayChart(720, 480, SeriesChartType.Point, "Filter kernel", "k", "h(k)");
            mydisplay2.AddData(myFilter.arg, myFilter.a);

            double[] dArg = new double[myFilter.Samplerate];  // double argument to display frequency response
            for (int k = 0; k < myFilter.Samplerate; k++)
            {
                dArg[k] = (double)k/(double)(8);//*myFilter.Samplerate);
            }
            DisplayChart mydisplay3 = new DisplayChart(720, 480, SeriesChartType.Line, "Real frequency response", "f", "|H(f)|");
            mydisplay3.AddData(dArg, myFilter.AS2);

            // ...and run the chart-application
            Application.EnableVisualStyles();
            Application.Run(mydisplay1);
            Application.Run(mydisplay2);
            Application.Run(mydisplay3);

            mydisplay1.Dispose();
            mydisplay2.Dispose();
            mydisplay3.Dispose();
            Application.Exit();

            for (int i = 0; i < 20; i++) Console.WriteLine();
        }
    }
}

